@@ -10,6 +10,7 @@
 
 extern 	unsigned long 	x_debug;
 	BUILT_IN_COMMAND(xdebugcmd);
+extern	char *	function_xdebug (char *);
 
 #define DEBUG_LOCAL_VARS	1 << 0
 #define DEBUG_ALIAS		1 << 1
@@ -35,5 +36,8 @@ extern 	unsigned long 	x_debug;
 #define DEBUG_AUTOKEY		1 << 21
 #define DEBUG_EXTRACTW		1 << 22
 #define DEBUG_SLASH_HACK	1 << 23
+#define DEBUG_SSL		1 << 24
+#define DEBUG_LASTLOG		1 << 25
+#define DEBUG_EXTRACTW_DEBUG	1 << 26
 
 #endif
