@@ -1,13 +1,43 @@
+/* $EPIC: term.c,v 1.12 2003/12/03 22:17:40 jnelson Exp $ */
 /*
  * term.c -- termios and (termcap || terminfo) handlers
  *
- * Copyright 1990 Michael Sandrof
- * Copyright 1995 Matthew Green
- * Coypright 1996 EPIC Software Labs
- * Copyright 1998 J. Kean Johnston, used with permission
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
+ * Copyright (c) 1990 Michael Sandroff.
+ * Copyright (c) 1991, 1992 Troy Rollo.
+ * Copyright (c) 1992-1996 Matthew Green.
+ * Copyright  1998 J. Kean Johnston, used with permission.
+ * Copyright  1995-2002 EPIC Software Labs 
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notices, the above paragraph (the one permitting redistribution),
+ *    this list of conditions and the following disclaimer in the 
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The names of the author(s) may not be used to endorse or promote 
+ *    products derived from this software without specific prior written
+ *    permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR 
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
+ * SUCH DAMAGE.
+ */
+/*
+ * Ben Winslow deserves specific praise for his fine work adding 
+ * terminfo support to EPIC.
  */
-
 #define __need_putchar_x__
 #define __need_term_flush__
 #include "irc.h"
@@ -34,15 +64,18 @@
 #endif
 #include <sys/ioctl.h>
 
-	int		need_redraw;
+volatile	int		need_redraw;
 static	int		tty_des;		/* descriptor for the tty */
 static	struct	termios	oldb, newb;
 	char		my_PC, *BC, *UP;
 	int		BClen, UPlen;
 
-#ifndef WTERM_C
-
-/* Systems cant seem to agree where to put these... */
+/*
+ * Systems can't seem to agree where to put these.
+ * System's can't seem to agree if (char *) is spelled (unsigned char *)
+ * or (const char *) or (const unsigned char *), so don't prototype these
+ * either.  You have been warned!
+ */
 #ifdef HAVE_TERMINFO
 extern	int		setupterm();
 extern	char		*tigetstr();
@@ -669,13 +702,13 @@ void	term_putchar (unsigned char c)
  * term_reset: sets terminal attributed back to what they were before the
  * program started 
  */
-void term_reset (void)
+void	term_reset (void)
 {
 	tcsetattr(tty_des, TCSADRAIN, &oldb);
 
 	if (current_term->TI_csr)
-		tputs_x(tparm(current_term->TI_csr, 0, current_term->TI_lines - 1));
-	term_gotoxy(0, current_term->TI_lines - 1);
+		tputs_x(tparm(current_term->TI_csr, 0, main_screen->li - 1));
+	term_gotoxy(0, main_screen->li - 1);
 #if use_alt_screen
 	if (current_term->TI_rmcup)
 		tputs_x(current_term->TI_rmcup);
@@ -717,7 +750,6 @@ void 	term_pause (char unused, char *not_used)
 	term_reset();
 	kill(getpid(), SIGSTOP);
 }
-#endif /* NOT IN WTERM_C */
 
 
 
@@ -732,7 +764,6 @@ int	termfeatures = 0;
 
 int 	term_init (void)
 {
-#ifndef WTERM_C
 	int	i,
 		desired;
 	char	*term;
@@ -751,7 +782,6 @@ int 	term_init (void)
 		fprintf(stdout, "Using terminal type [%s]\n", term);
 
 #ifdef HAVE_TERMINFO
-	tptr = termcap, tptr = termcap2;	/* Shut up gcc */
 	setupterm(NULL, 1, &i);
 	if (i != 1)
 	{
@@ -788,14 +818,14 @@ int 	term_init (void)
 		{
 			cval = Tgetstr(tcaps[i], tptr);
 			if (cval == (char *) -1)
-				*(char **)tcaps[i].ptr = NULL;
+				*(char * *)tcaps[i].ptr = NULL;
 			else
-				*(char **)tcaps[i].ptr = cval;
+				*(char * *)tcaps[i].ptr = cval;
 		}
 	}
 
-	BC = current_term->TI_cub1;
-	UP = current_term->TI_cuu1;
+	BC = malloc_strdup(current_term->TI_cub1);
+	UP = malloc_strdup(current_term->TI_cuu1);
 	if (current_term->TI_pad)
 		my_PC = current_term->TI_pad[0];
 	else
@@ -824,23 +854,29 @@ int 	term_init (void)
 
 	current_term->TI_normal[0] = 0;
 	if (current_term->TI_sgr0)
-	{
-		strcat(current_term->TI_normal, current_term->TI_sgr0);
-	}
+		strlcat(current_term->TI_normal, current_term->TI_sgr0, sizeof current_term->TI_normal);
 	if (current_term->TI_rmso)
 	{
-		if (current_term->TI_sgr0 && strcmp(current_term->TI_rmso, current_term->TI_sgr0))
-			strcat(current_term->TI_normal, current_term->TI_rmso);
+		if (current_term->TI_sgr0 && 
+		    strcmp(current_term->TI_rmso, current_term->TI_sgr0))
+			strlcat(current_term->TI_normal, current_term->TI_rmso, sizeof current_term->TI_sgr0);
 	}
 	if (current_term->TI_rmul)
 	{
-		if (current_term->TI_sgr0 && strcmp(current_term->TI_rmul, current_term->TI_sgr0))
-			strcat (current_term->TI_normal, current_term->TI_rmul);
+		if (current_term->TI_sgr0 && 
+		    strcmp(current_term->TI_rmul, current_term->TI_sgr0))
+			strlcat(current_term->TI_normal, current_term->TI_rmul, sizeof current_term->TI_normal);
 	}
+	/*
+	 * On some systems, alternate char set mode isn't exited with sgr0
+	 */
+	if (current_term->TI_rmacs)
+		strlcat(current_term->TI_normal, current_term->TI_rmacs, sizeof current_term->TI_normal);
 
 
 	/*
-	 * Finally set up the current_term->TI_sgrstrs array.  Clean it out first...
+	 * Finally set up the current_term->TI_sgrstrs array.
+	 * Clean it out first...
 	 */
 	for (i = 0; i < TERM_SGR_MAXVAL; i++)
 		current_term->TI_sgrstrs[i] = "";
@@ -973,46 +1009,26 @@ int 	term_init (void)
 	{
 		char cbuf[128];
 
-		cbuf[0] = '\0';
-		if (i >= 8)
-			strcpy(cbuf, current_term->TI_sgrstrs[TERM_SGR_BOLD_ON-1]);
-#if 0
-		else
-			strcpy(cbuf, current_term->TI_sgrstrs[TERM_SGR_BOLD_OFF-1]);
-#endif
-
+		*cbuf = 0;
 		if (current_term->TI_setaf) 
-			strcat(cbuf, tparm(current_term->TI_setaf, i & 0x07, 0));
+		    strlcat(cbuf, tparm(current_term->TI_setaf, i & 0x07, 0), sizeof cbuf);
 		else if (current_term->TI_setf)
-			strcat(cbuf, tparm(current_term->TI_setf, i & 0x07, 0));
-		else if (i >= 8)
-			sprintf(cbuf, "\033[1;%dm", (i & 0x07) + 30);
+		    strlcat(cbuf, tparm(current_term->TI_setf, i & 0x07, 0), sizeof cbuf);
 		else
-			sprintf(cbuf, "\033[%dm", (i & 0x07) + 30);
+		    snprintf(cbuf, sizeof cbuf, "\033[%dm", (i & 0x07) + 30);
 
-		current_term->TI_forecolors[i] = m_strdup(cbuf);
-	}
-            
-	for (i = 0; i < 16; i++)
-	{
-		char cbuf[128];
-
-		cbuf[0] = '\0';
-		if (i >= 8)
-			strcpy(cbuf, current_term->TI_sgrstrs[TERM_SGR_BLINK_ON - 1]);
+		current_term->TI_forecolors[i] = malloc_strdup(cbuf);
 
+		*cbuf = 0;
 		if (current_term->TI_setab)
-			strcat (cbuf, tparm(current_term->TI_setab, i & 0x07, 0));
+		    strlcat(cbuf, tparm(current_term->TI_setab, i & 0x07, 0), sizeof cbuf);
 		else if (current_term->TI_setb)
-			strcat (cbuf, tparm(current_term->TI_setb, i & 0x07, 0));
-		else if (i >= 8)
-			sprintf(cbuf, "\033[1;%dm", (i & 0x07) + 40);
+		    strlcat(cbuf, tparm(current_term->TI_setb, i & 0x07, 0), sizeof cbuf);
 		else
-			sprintf(cbuf, "\033[%dm", (i & 0x07) + 40);
+		    snprintf(cbuf, sizeof cbuf, "\033[%dm", (i & 0x07) + 40);
 
-		current_term->TI_backcolors[i] = m_strdup(cbuf);
+		current_term->TI_backcolors[i] = malloc_strdup(cbuf);
 	}
-#endif /* NOT IN WTERM_C */
 
 /* Set up the terminal discipline */
 	tty_des = 0;			/* Has to be. */
@@ -1020,6 +1036,13 @@ int 	term_init (void)
 
 	newb = oldb;
 	newb.c_lflag &= ~(ICANON | ECHO); /* set equ. of CBREAK, no ECHO */
+#ifdef IEXTEN
+	if (use_iexten == 1)
+		newb.c_lflag |= IEXTEN;	    /* Turn off DISCARD/LNEXT */
+	else if (use_iexten == 0)
+		newb.c_lflag &= ~(IEXTEN);  /* Turn off DISCARD/LNEXT */
+#endif
+
 	newb.c_cc[VMIN] = 1;	          /* read() satified after 1 char */
 	newb.c_cc[VTIME] = 0;	          /* No timer */
 
@@ -1031,6 +1054,10 @@ int 	term_init (void)
 #               endif
 #       endif
 
+	/* 
+	 * We can't just turn off ISIG since we want ^C to send SIGINT,
+	 * so we turn off all of the other signal-generating keys
+	 */
 	newb.c_cc[VQUIT] = _POSIX_VDISABLE;
 #	ifdef VDSUSP
 		newb.c_cc[VDSUSP] = _POSIX_VDISABLE;
@@ -1039,7 +1066,6 @@ int 	term_init (void)
 		newb.c_cc[VSUSP] = _POSIX_VDISABLE;
 #	endif
 
-#ifndef WTERM_C
 	if (!use_flow_control)
 		newb.c_iflag &= ~IXON;	/* No XON/XOFF */
 
@@ -1051,7 +1077,6 @@ int 	term_init (void)
 #if use_automargins
 	if (current_term->TI_rmam)
 		tputs_x(current_term->TI_rmam);
-#endif
 #endif
 
 	tcsetattr(tty_des, TCSADRAIN, &newb);
@@ -1059,14 +1084,13 @@ int 	term_init (void)
 }
 
 
-#ifndef WTERM_C
 /*
  * term_resize: gets the terminal height and width.  Trys to get the info
  * from the tty driver about size, if it can't... uses the termcap values. If
  * the terminal size has changed since last time term_resize() has been
  * called, 1 is returned.  If it is unchanged, 0 is returned. 
  */
-int term_resize (void)
+int	term_resize (void)
 {
 	static	int	old_li = -1,
 			old_co = -1;
@@ -1097,9 +1121,7 @@ int term_resize (void)
 
 #if use_automargins
 	if (!current_term->TI_am || !current_term->TI_rmam)
-	{
 		current_term->TI_cols--;
-	}
 #else
 	current_term->TI_cols--;
 #endif
@@ -1112,9 +1134,9 @@ int term_resize (void)
 			main_screen->li = current_term->TI_lines;
 		if (main_screen)
 			main_screen->co = current_term->TI_cols;
-		return (1);
+		return 1;
 	}
-	return (0);
+	return 0;
 }
 
 
@@ -1135,7 +1157,7 @@ void	term_beep (void)
 	}
 }
 
-int	orig_term_eight_bit (void)
+static int	orig_term_eight_bit (void)
 {
 	if (dumb_mode)
 		return 1;
@@ -1167,8 +1189,10 @@ void	set_term_eight_bit (int value)
 	tcsetattr(tty_des, TCSADRAIN, &newb);
 }
 
-void	set_meta_8bit (int value)
+void	set_meta_8bit (const void *stuff)
 {
+	int	value = *(const int *)stuff;
+
 	if (dumb_mode)
 		return;
 
@@ -1246,7 +1270,9 @@ void	term_clrscr (void)
  */
 void	term_left (int num)
 {
-	if (current_term->TI_cub)
+	if (num == 1 && current_term->TI_cub1)
+		tputs_x(current_term->TI_cub1);
+	else if (current_term->TI_cub)
 		tputs_x (tparm(current_term->TI_cub, num));
 	else if (current_term->TI_mrcup)
 		tputs_x (tparm(current_term->TI_mrcup, -num, 0));
@@ -1263,7 +1289,9 @@ void	term_left (int num)
  */
 void	term_right (int num)
 {
-	if (current_term->TI_cuf)
+	if (num == 1 && current_term->TI_cuf1)
+		tputs_x(current_term->TI_cuf1);
+	else if (current_term->TI_cuf)
 		tputs_x (tparm(current_term->TI_cuf, num));
 	else if (current_term->TI_mrcup)
 		tputs_x (tparm(current_term->TI_mrcup, num, 0));
@@ -1354,8 +1382,8 @@ void	term_scroll (int top, int bot, int n)
 		 * region was the full screen.  That test *always* fails,
 		 * because we never scroll the bottom line of the screen.
 		 */
-		strcpy(start, tparm(current_term->TI_csr, top, bot));
-		strcpy(final, tparm(current_term->TI_csr, 0, current_term->TI_lines-1));
+		strlcpy(start, tparm(current_term->TI_csr, top, bot), sizeof start);
+		strlcpy(final, tparm(current_term->TI_csr, 0, current_term->TI_lines-1), sizeof final);
 
 		if (n > 0)
 		{
@@ -1364,10 +1392,10 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_indn)
 			{
 				oneshot = 1;
-				strcpy(thing, tparm(current_term->TI_indn, rn, rn));
+				strlcpy(thing, tparm(current_term->TI_indn, rn, rn), sizeof thing);
 			}
 			else
-				strcpy(thing, current_term->TI_ind);
+				strlcpy(thing, current_term->TI_ind, sizeof thing);
 		}
 		else
 		{
@@ -1376,17 +1404,17 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_rin)
 			{
 				oneshot = 1;
-				strcpy (thing, tparm(current_term->TI_rin, rn, rn));
+				strlcpy(thing, tparm(current_term->TI_rin, rn, rn), sizeof thing);
 			}
 			else
-				strcpy (thing, current_term->TI_ri);
+				strlcpy(thing, current_term->TI_ri, sizeof thing);
 		}
 	}
 
 	else if (current_term->TI_wind && (current_term->TI_ri || current_term->TI_rin) && (current_term->TI_ind || current_term->TI_indn))
 	{
-		strcpy(start, tparm(current_term->TI_wind, top, bot, 0, current_term->TI_cols-1));
-		strcpy(final, tparm(current_term->TI_wind, 0, current_term->TI_lines-1, 0, current_term->TI_cols-1));
+		strlcpy(start, tparm(current_term->TI_wind, top, bot, 0, current_term->TI_cols-1), sizeof start);
+		strlcpy(final, tparm(current_term->TI_wind, 0, current_term->TI_lines-1, 0, current_term->TI_cols-1), sizeof final);
 
 		if (n > 0)
 		{
@@ -1395,10 +1423,10 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_indn)
 			{
 				oneshot = 1;
-				strcpy (thing, tparm(current_term->TI_indn, rn, rn));
+				strlcpy(thing, tparm(current_term->TI_indn, rn, rn), sizeof thing);
 			}
 			else
-				strcpy (thing, current_term->TI_ind);
+				strlcpy(thing, current_term->TI_ind, sizeof thing);
 		}
 		else
 		{
@@ -1407,10 +1435,10 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_rin)
 			{
 				oneshot = 1;
-				strcpy (thing,tparm(current_term->TI_rin, rn, rn));
+				strlcpy(thing, tparm(current_term->TI_rin, rn, rn), sizeof thing);
 			}
 			else
-				strcpy (thing, current_term->TI_ri);
+				strlcpy(thing, current_term->TI_ri, sizeof thing);
 		}
 	}
 
@@ -1424,18 +1452,18 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_dl)
 			{
 				oneshot = 1;
-				strcpy (thing, tparm(current_term->TI_dl, rn, rn));
+				strlcpy(thing, tparm(current_term->TI_dl, rn, rn), sizeof thing);
 			}
 			else
-				strcpy (thing, current_term->TI_dl1);
+				strlcpy(thing, current_term->TI_dl1, sizeof thing);
 
 			if (current_term->TI_il)
 			{
 				oneshot = 1;
-				strcpy(final, tparm(current_term->TI_il, rn, rn));
+				strlcpy(final, tparm(current_term->TI_il, rn, rn), sizeof final);
 			}
 			else
-				strcpy(final, current_term->TI_il1);
+				strlcpy(final, current_term->TI_il1, sizeof final);
 		}
 		else
 		{
@@ -1444,18 +1472,18 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_il)
 			{
 				oneshot = 1;
-				strcpy (thing, tparm(current_term->TI_il, rn, rn));
+				strlcpy(thing, tparm(current_term->TI_il, rn, rn), sizeof thing);
 			}
 			else
-				strcpy (thing, current_term->TI_il1);
+				strlcpy(thing, current_term->TI_il1, sizeof thing);
 
 			if (current_term->TI_dl)
 			{
 				oneshot = 1;
-				strcpy(final, tparm(current_term->TI_dl, rn, rn));
+				strlcpy(final, tparm(current_term->TI_dl, rn, rn), sizeof thing);
 			}
 			else
-				strcpy(final, current_term->TI_dl1);
+				strlcpy(final, current_term->TI_dl1, sizeof thing);
 		}
 	}
 
@@ -1517,9 +1545,9 @@ void	term_scroll (int top, int bot, int n)
  * 14   - bright cyan (foreground only)
  * 15   - bright white (foreground only)
  */
-char *	term_getsgr (int opt, int fore, int back)
+const char *	term_getsgr (int opt, int fore, int back)
 {
-	char *ret = empty_string;
+	const char *ret = empty_string;
 
 	switch (opt)
 	{
@@ -1552,7 +1580,7 @@ char *	term_getsgr (int opt, int fore, int back)
 	return (ret);
 }
 
-char *	control_mangle (unsigned char *text)
+static char *	control_mangle (unsigned char *text)
 {
 static 	u_char	retval[256];
 	int 	pos = 0;
@@ -1581,7 +1609,7 @@ static 	u_char	retval[256];
 	return retval;
 }
 
-char *	get_term_capability (char *name, int querytype, int mangle)
+const char *	get_term_capability (const char *name, int querytype, int mangle)
 {
 static	char		retval[128];
 	const char *	compare = empty_string;
@@ -1606,14 +1634,14 @@ static	char		retval[128];
 			case CAP_TYPE_INT:
 				if (!(int *)t->ptr)
 					return NULL;
-				strcpy(retval, ltoa(* (int *)(t->ptr)));
+				strlcpy(retval, ltoa(* (int *)(t->ptr)), sizeof retval);
 				return retval;
-		        case CAP_TYPE_STR:
+			case CAP_TYPE_STR:
 				if (!(char **)t->ptr || !*(char **)t->ptr)
 					return NULL;
-				strcpy(retval, mangle ? 
+				strlcpy(retval, mangle ? 
 					control_mangle(*(char **)t->ptr) :
-					(*(char **)t->ptr));
+						      (*(char **)t->ptr), sizeof retval);
 				return retval;
 		    }
 		}
@@ -1621,4 +1649,3 @@ static	char		retval[128];
 	return NULL;	
 }
 
-#endif /* NOT IN WTERM_C */
